%% THR = decisionBoundaryCarving (POS, NEG, RANGE, TYPE)
%
%  Decision boundary carving function for binary SVM classification
%
%  POS: DEC_VALUE (from LibSVM) of positive features (class 1)
%  NEG: DEV_VALUE (from LibSVM) of negative features (class -1)
%  ACCRTR: Relative accuracy obtained on training step (training dataset)
%  RANGE: increment of the threshold (standard value: 0.0001)
%  TYPE:
%    'restrict' : THRESHOLD goes from the highest to the lowest decision value (standard)
%    'relax' : THRESHOLD goes from the lowest to the highest decision value
%    'average' : THRESHOLD is an average of 'relax' and 'restrict' results
%%
function [THR, AccR] = decisionBoundaryCarving(POS, NEG, ACCRTR, RANGE, TYPE)
    if strcmp(TYPE, 'restrict')
        [THR, AccR] = DBC_MAX_TO_MIN(POS, NEG, RANGE, ACCRTR);
    elseif strcmp(TYPE, 'relax')
        [THR, AccR] = DBC_MIN_TO_MAX(POS, NEG, RANGE, ACCRTR);
    else
        [T1, AccR1] = DBC_MIN_TO_MAX(POS, NEG, RANGE, ACCRTR);
        [T2, AccR2] = DBC_MAX_TO_MIN(POS, NEG, RANGE, ACCRTR);
        THR = (T1 + T2)/2.0;
        AccR = (AccR1 + AccR2)/2;
    end
end

%%
function [THR_MIN_TO_MAX, ACCR] = DBC_MIN_TO_MAX(POS, NEG, RANGE, AccRTR)
    %Defining Minimum and Maximum decision values
    MIN = min(cat(1,POS,NEG));
    MAX = max(cat(1,POS,NEG));

    %Accuracy
    ACCR=AccRTR;
    %Epslon
    E = MIN;
    %THRESHOLD (RELAX CASE)
    THR_MIN_TO_MAX = 0;
    while (E <= MAX)
        RESP = (((sum(POS >= E)/length(POS)))+(sum(NEG < E)/length(NEG)))/2;
        %If the new relative accuracy is higher than the old relative accuracy
        if RESP > ACCR
            %Updating the threshold
            THR_MIN_TO_MAX = E;
            %Updating accuracy
            ACCR = RESP;
        end
        %Increase epslon
        E = E + RANGE;
    end
end

%%
function [THR_MAX_TO_MIN, ACCR] = DBC_MAX_TO_MIN(POS, NEG, RANGE, AccRTR)

    %Defining Minimum and Maximum decision values
    MIN = min(cat(1,POS,NEG));
    MAX = max(cat(1,POS,NEG));

    %Accuracy
    ACCR=AccRTR;
    %Epslon
    E = MAX;
    %THRESHOLD (RELAX CASE)
    THR_MAX_TO_MIN = 0;
    while (E >= MIN)
        RESP = (((sum(POS >= E)/length(POS)))+(sum(NEG<E)/length(NEG)))/2;
        %If the new relative accuracy is higher than the old relative accuracy
        if RESP > ACCR
            %Update the threshold
            THR_MAX_TO_MIN = E;
            ACCR = RESP;
        end
        %Decrease epslon
        E = E - RANGE;
    end
end